@@ -10,6 +10,7 @@ from glob import glob
 import datetime
 import pandas
 import sigfig
+from pprint import pprint
 import json
 import os
 import re
@@ -41,6 +42,7 @@ class ROSSPkg():
         self.results = {}
         self.results['figures'] = {}
         self.verbose = verbose
+        self.jupyter = jupyter
         self.plot_title = None
         if operating_system == 'mac':
             import phreeqpy.iphreeqc.phreeqc_dll as phreeqc_mod
@@ -111,6 +113,8 @@ class ROSSPkg():
 
         # calculate module properties
         self.parameters['repeated_membrane_winding_mm'] = 2 * self.parameters['membrane_thickness_mm'] + self.parameters['feed_thickness_mm'] + self.parameters['permeate_thickness_mm'] + 2 * self.parameters['polysulfonic_layer_thickness_mm'] + 2 * self.parameters['support_layer_thickness_mm']     
+        if self.parameters['repeated_membrane_winding_mm'] == 0:
+            print('--> ERROR: The module dimensions are not sensible.')
         self.parameters['cells_per_module'] = cells_per_module
         self.variables['cell_meters'] = self.parameters['module_length_m'] / self.parameters['cells_per_module']        
         self.parameters['active_m2_cell'] = self.parameters['active_m2'] / self.parameters['cells_per_module']
@@ -159,7 +163,7 @@ class ROSSPkg():
         lengths_line = '-lengths\t\t{}'.format(self.variables['cell_meters'])
         timestep_line = '-time_step\t\t{}\t# this satisfies the Courant condition with a feed velocity of {} m/s'.format(self.parameters['timestep'], sigfigs_conversion(feed_velocity, 4))
         initial_time_line = '-initial_time\t\t0'    
-        boundary_conditions_line = '-boundary_conditions\tconstant\tconstant \t # Dirichlet boundary condition'
+        boundary_conditions_line = '-boundary_conditions\tconstant\tflux \t # Dirichlet and Cachy boundary conditions'
         
         # define the domain-dependent parameters
         domain_line = ''
@@ -197,8 +201,11 @@ class ROSSPkg():
             print('permeate_removal_per_cell', self.parameters['permeate_moles_per_cell'])
             print('active_cm_squared_cell', (self.parameters['active_m2_cell'] / centi**2))
 
-    def reaction(self, permeate_approach = 'linear_permeate', permeate_efficiency = 1, head_loss = -0.15, final_cf = 2):
-        '''Define the REACTION block'''        
+    def reaction(self, permeate_approach = 'linear_permeate', permeate_efficiency = 1, head_loss = 0.89, final_cf = 2):
+        '''
+        Define the REACTION block
+            The head_loss parameter default of 11% is sourced from “Reverse osmosis desalination: Modeling and experiment” by Fraidenraich et al.; 2009.
+        '''        
         # establish parameters
         self.parameters['permeate_approach'] = permeate_approach
         
@@ -206,21 +213,26 @@ class ROSSPkg():
         cell_moles = []
         reaction_parameters = []
         self.results['reaction_block'] = []
-        cumulative_cf = 1 
+        self.cumulative_cf = 1 
         for module in range(self.parameters['quantity_of_modules']): 
             if permeate_approach == 'linear_permeate':
-                initial_moles_removed = self.parameters['permeate_moles_per_cell'] * 2 / (1 + exp(head_loss))
-                final_moles_removed = initial_moles_removed * exp(head_loss)
+                initial_moles_removed = self.parameters['permeate_moles_per_cell'] * 2 / (1 + head_loss)
+                final_moles_removed = initial_moles_removed * head_loss
                 removed_moles_slope = ((final_moles_removed - initial_moles_removed) / (self.parameters['cells_per_module'])) / permeate_efficiency
                 average_moles_removed = (final_moles_removed + initial_moles_removed) / 2
+                module_parameters = []
 
                 for cell in range(self.parameters['cells_per_module']):
                     removed_moles_in_cell = (cell * removed_moles_slope + initial_moles_removed)
                     reaction_parameters.append(removed_moles_in_cell)
+                    module_parameters.append(removed_moles_in_cell)
                     
                 moles_removed = sum(reaction_parameters)
-                cf = self.variables['feed_moles'] / (self.variables['feed_moles'] - moles_removed)
-                self.variables[f'module {module}'] = {'cumulative_cf': cf, 'permeate (moles/cell)': removed_moles_slope}
+                self.cumulative_cf = self.variables['feed_moles'] / (self.variables['feed_moles'] - moles_removed)
+                module_moles_removed = sum(module_parameters)
+                cf = self.variables['feed_moles'] / (self.variables['feed_moles'] - module_moles_removed)
+                
+                self.variables[f'module {module}'] = {'cf': cf, 'permeate (moles/cell)': removed_moles_slope}
                 moles_remaining = self.variables['feed_moles'] - moles_removed
                 
                 if sigfigs_conversion(average_moles_removed,14) != sigfigs_conversion(self.parameters['permeate_moles_per_cell'], 14):
@@ -234,7 +246,7 @@ class ROSSPkg():
                     print('moles_remaining', moles_remaining)
                     print('moles_removed', moles_removed)
 
-            if permeate_approach == 'linear_cf':
+            elif permeate_approach == 'linear_cf':
                 module_iteration = module_previous_moles_removed = moles_removed = 0
                 initial_cf = 1
 
@@ -257,8 +269,9 @@ class ROSSPkg():
                     module_iteration += 1
 
                 cf = cfs[-1]
-                cumulative_cf *= cf
+                self.cumulative_cf *= cf
                 moles_remaining = self.variables['feed_moles'] - moles_removed  
+                self.variables[f'module {module}'] = {'cf': cf}
 
             if self.parameters['simulation_type'] == 'transport':
                 self.results['reaction_block'].append('\n')
@@ -274,8 +287,7 @@ class ROSSPkg():
                     if cell < self.parameters['cells_per_module']:
                         reaction_line += f'\n\tH2O -1; {reaction_parameters[reaction_index]}' 
                     elif cell == self.parameters['cells_per_module']:
-                        reaction_line += f'''\n\tH2O -1; {reaction_parameters[reaction_index]}
-        INCREMENTAL_REACTIONS \ttrue'''   
+                        reaction_line += f'''\n\tH2O -1; {reaction_parameters[reaction_index]}'''   
                            
                     self.results['reaction_block'].append(reaction_line)
 
@@ -298,28 +310,26 @@ class ROSSPkg():
 
             # the calculated reaction parameters will be added and printed to a generated PHREEQC input file
             final_solution_mass = moles_remaining * self.parameters['water_mw'] * milli  #kg water mass
-            final_cf_cell = self.variables['feed_kg'] / final_solution_mass
-
             if self.parameters['os'] == 'windows':
                 self.results['reaction_block'].append('#%s' %(permeate_approach))
                 if permeate_approach == 'linear permeate':
                     self.results['reaction_block'].append(f'''
         #Permeate efficiency parameter: {permeate_efficiency}
-        #Head loss parameter: {head_loss}''')
+        #Effluent relative pressure: {head_loss}''')
 
                 self.results['reaction_block'].append(f'''    #Effluent module {module + 1}:
         #Estimated CF: {sigfigs_conversion(cf, 4)}
-        #Estimated solution mass: {final_solution_mass}\n\n''')
+        #Estimated final water mass: {final_solution_mass}\n\n''')
 
             if self.verbose:
-                print('Effluent module %s CF:' %(module + 1), final_cf_cell)
+                print(f'Effluent module {module + 1} CF: {cf}')
 
-    def solutions(self, water_selection = '', custom_water_parameters = {}, solution_description = '', parameterized_alkalinity = False, parameterized_ph_charge = True):
+    def solutions(self, water_selection = '', water_characteristics = {}, solution_description = '', parameterized_alkalinity = False, parameterized_ph_charge = True):
         """Specify the SOLUTION block of the simulation."""
         # create the solution line of the input file
         self.results['solution_block'] = []
         
-        if water_selection == '' and custom_water_parameters == {}:
+        if (water_selection == '' or water_selection == 'custom') and water_characteristics == {}:
             print('--> ERROR: Feed geochemistry is not provided.')
         
         self.parameters['water_selection'] = water_selection
@@ -335,26 +345,34 @@ class ROSSPkg():
 
         #=============================================================================
         # determine which predefined solution should be simulated
-
-        elements_lines = []
+        
+        def define_elements():
+            elements_lines = []
+            predicted_effluent = {}
+            for element, information2 in information.items():
+                self.parameters['solution_elements'].append(element)
+                conc = information2['concentration (ppm)']
+                predicted_effluent[element] = conc*self.cumulative_cf
+                ref = information2['reference']
+                if element in self.elements:                          
+                    if len(str(conc)) <= 3:
+                        elements_lines.append(f'{element}\t\t{conc}\t#{ref}')
+                    else:
+                        elements_lines.append(f'{element}\t{conc}\t#{ref}')
+                else:
+                    print('\n--> ERROR: The {} element is not accepted by the {} database'.format(element, self.parameters['database_selection']))
+            return elements_lines, predicted_effluent
+            
         self.parameters['solution_elements'] = []
         temperature = ph = alkalinity = pe = None
-        if water_selection != '':       
+        if water_selection not in ['', 'custom']:       
             # import the predefined water body
             water_file_path = os.path.join(self.parameters['root_path'], 'water_bodies', f'{water_selection}.json')
             water_body = json.load(open(water_file_path))
             
             for content, information in water_body.items():
                 if content == 'element':
-                    for element, information2 in information.items():
-                        if element in self.elements:
-                            self.parameters['solution_elements'].append(element)
-                            conc = information2['concentration (ppm)']
-                            ref = information2['reference']
-                            elements_lines.append(f'{element}\t{conc}\t#{ref}')
-                        else:
-                            print('\n--> ERROR: The {} element is not accepted by the {} database'.format(element, self.parameters['database_selection']))
-                                
+                    elements_lines, self.predicted_effluent = define_elements()                                
                 elif content == 'temperature':
                     temperature = information['celcius']
                     temperature_reference = information['reference']
@@ -368,34 +386,23 @@ class ROSSPkg():
                     ph = information['value']
                     ph_reference = information['reference']
 
-        if custom_water_parameters != {}:
-            for content, information in custom_water_parameters.items():
+        if water_characteristics != {}:
+            for content, information in water_characteristics.items():
                 if content == 'element':
-                    for element, information2 in information.items():
-                        self.parameters['solution_elements'].append(element)
-                        conc = information2['concentration (ppm)']
-                        ref = information2['reference']
-                        if element in self.elements:                          
-                            if len(str(conc)) <= 3:
-                                elements_lines.append(f'{element}\t\t{conc}\t#{ref}')
-                            else:
-                                elements_lines.append(f'{element}\t{conc}\t#{ref}')
-                        else:
-                            print('\n--> ERROR: The {} element is not accepted by the {} database'.format(element, self.parameters['database_selection']))
-                                    
+                    elements_lines, self.predicted_effluent = define_elements()
                 # create the temperature line of the input file
                 elif content == 'temperature':                    
-                    temperature = custom_water_parameters['temperature']['value']
-                    temperature_reference = custom_water_parameters['temperature']['reference']
+                    temperature = water_characteristics['temperature']['value']
+                    temperature_reference = water_characteristics['temperature']['reference']
                 elif content == 'pe':       
-                    pe = custom_water_parameters['pe']['value']
-                    pe_reference = custom_water_parameters['pe']['reference']
+                    pe = water_characteristics['pe']['value']
+                    pe_reference = water_characteristics['pe']['reference']
                 elif content == 'Alkalinity':
-                    alkalinity = custom_water_parameters['Alkalinity']['value']
-                    alkalinity_reference = custom_water_parameters['Alkalinity']['reference'] 
+                    alkalinity = water_characteristics['Alkalinity']['value']
+                    alkalinity_reference = water_characteristics['Alkalinity']['reference'] 
                 elif content == 'pH':
-                    ph = custom_water_parameters['ph']['value']
-                    ph_reference = custom_water_parameters['ph']['reference']
+                    ph = water_characteristics['pH']['value']
+                    ph_reference = water_characteristics['pH']['reference']
                     
         # parameterize the lines of the SOLUTIONS block
         temperature_line = ''
@@ -411,9 +418,9 @@ class ROSSPkg():
             if parameterized_ph_charge and not parameterized_alkalinity:
                 ph_line = f'pH \t\t {ph} charge #{ph_reference}'
                 alkalinity_line = ''      
-            elif parameterized_alkalinity and not parameterized_ph_charge:
-                if alkalinity is not None:
-                    ph_line = f'pH \t\t {ph} #{ph_reference}'
+            elif not parameterized_ph_charge:
+                ph_line = f'pH \t\t {ph} #{ph_reference}'
+                if parameterized_alkalinity and alkalinity is not None:
                     alkalinity_line = f'Alkalinity \t {alkalinity} #{alkalinity_reference}'
                    
         unit_line = 'units \t ppm' 
@@ -433,22 +440,10 @@ class ROSSPkg():
             if self.parameters['domain'] == 'dual':
                 total_cells = total_cells*2+1
             feed_solution_line = f'\nSOLUTION 1-{total_cells}\tInitial solution in the RO module'
-            self.results['solution_block'].extend([feed_solution_line,'temp \t 25','units \t ppm'])
-
-            for element in self.parameters['solution_elements']:
-                element_line = f'{element}\t0'    
-                self.results['solution_block'].append(element_line)
-
             water_line = '-water \t{}'.format(self.variables['feed_kg'])
-            self.results['solution_block'].append(water_line)
+            self.results['solution_block'].extend([feed_solution_line,'temp \t 25','units \t ppm',water_line])
 
-
-    def equilibrium_phases(self, block_comment = '', ignored_minerals = [], existing_parameters = {}):
-        """Specify the EQUILIBRIUM_PHASES block of the simulation."""
-        # define mineral sizes for later spacing
-        short_mineral_names = ['Barite', 'Gypsum','Halite', 'Natron', 'Quartz', 'Talc','Trona', 'Borax', 'Albite', 'K-mica','Illite', 'Pyrite', 'Sulfur',]
-        long_mineral_names = ['Anthophyllite', 'Hexahydrite', 'Leonhardite', 'Nesquehonite', 'Pentahydrite', 'Portlandite','Sepiolite(d)', 'Boric_acid,s', 'K2B4O7:4H2O', 'NaB5O8:5H2O', 'Rhodochrosite', 'Strontianite','Hydroxyapatite', 'Chlorite(14A)', 'Mackinawite', 'Hausmannite', 'Pyrochroite']
-        
+    def described_minerals(self,):
         # determine the set of possible minerals 
         self.variables['described_minerals'] = {}
         for mineral in self.minerals:
@@ -468,7 +463,14 @@ class ROSSPkg():
                 elif element not in self.parameters['solution_elements']:
                     if self.verbose:
                         print('--> The {} element is absent in the solution to describe the mineral < {} / {} >.'.format(element, original_formula, mineral))
-                        
+            
+
+    def equilibrium_phases(self, block_comment = '', ignored_minerals = [], existing_parameters = {}):
+        """Specify the EQUILIBRIUM_PHASES block of the simulation."""
+        # define mineral sizes for later spacing
+        short_mineral_names = ['Barite', 'Gypsum','Halite', 'Natron', 'Quartz', 'Talc','Trona', 'Borax', 'Albite', 'K-mica','Illite', 'Pyrite', 'Sulfur',]
+        long_mineral_names = ['Anthophyllite', 'Hexahydrite', 'Leonhardite', 'Nesquehonite', 'Pentahydrite', 'Portlandite','Sepiolite(d)', 'Boric_acid,s', 'K2B4O7:4H2O', 'NaB5O8:5H2O', 'Rhodochrosite', 'Strontianite','Hydroxyapatite', 'Chlorite(14A)', 'Mackinawite', 'Hausmannite', 'Pyrochroite']
+
         # define the equilibrium_phases block
         self.results['equilibrium_phases_block'] = []
         if self.parameters['simulation_type'] == 'transport':
@@ -483,6 +485,7 @@ class ROSSPkg():
         self.results['equilibrium_phases_block'].append(equilibrium_phases_line)
 
         # define the equilibrium_phases lines for the code block
+        self.described_minerals()
         for possible_mineral in self.variables['described_minerals']:
             if possible_mineral not in ignored_minerals:
                 if possible_mineral in short_mineral_names:
@@ -506,10 +509,9 @@ class ROSSPkg():
                 self.results['equilibrium_phases_block'].append(mineral_line)     
                 
         if self.verbose:
-            print(self.variables['described_minerals'])
+            pprint(self.variables['described_minerals'])
             
-    def selected_output(self, output_filename = None):
-        '''Specify the output file after a PHREEQC simulation'''
+    def selected_output_file_name(self,output_filename):
         # create parameter lines             
         if output_filename is None:
             count = 0
@@ -521,34 +523,42 @@ class ROSSPkg():
             selected_output_file_name = output_filename
 
         selected_output_file_name += '.txt'
+        
+        return selected_output_file_name
+            
+    def selected_output(self, output_filename = None):
+        '''Specify the output file after a PHREEQC simulation'''
+        self.parameters['selected_output_file_name'] = self.selected_output_file_name(output_filename)
 
         minerals_line = ' '.join([mineral for mineral in self.variables['described_minerals']])
         elements_line = ' '.join([element for element in self.parameters['solution_elements']])
 
         # define parameter lines
         first_line = '\nSELECTED_OUTPUT'
-        file_name_line = f'-file\t\t\t{selected_output_file_name}'
+        file_name_line = '-file\t\t\t{}'.format(self.parameters['selected_output_file_name'])
         reaction_line = '-reaction\t\ttrue'
         temperature_line = '-temperature\t\ttrue'
         total_elements_line = '-totals\t\t\t' + elements_line    
         saturation_indices_line = f'-saturation_indices\t{minerals_line}'
         equilibrium_phases_line = f'-equilibrium_phases\t{minerals_line}'
         ph_line = '-pH\t\t\ttrue'
+        alkalinity_line ='-alkalinity\ttrue' 
         solution_line = '-solution'
         time_line = '-time\t\t\ttrue'
         distance_line = '-distance\t\ttrue'
         simulation_line = '-simulation\t\ttrue'
         high_precision_line = '-high_precision\ttrue'
+        charge_balance_line = '-charge_balance\ttrue'
+        ionic_strength_line = '-ionic_strength\ttrue'
         step_line = '-step'
         water_line = '-water'
 
         # establish the selected_output_block
         self.results['selected_output_block'] = []
-        self.results['selected_output_block'].extend((first_line, file_name_line, reaction_line, temperature_line, total_elements_line, saturation_indices_line, equilibrium_phases_line, ph_line, time_line, distance_line, simulation_line, high_precision_line, solution_line, step_line,water_line))
-
-    def export(self, simulation_name = None, input_path = None, output_path = None, external_file = False):
-        """View and export the PHREEQC input file"""        
-        # define the simulation input path 
+        self.results['selected_output_block'].extend((first_line, file_name_line, reaction_line, temperature_line, total_elements_line, saturation_indices_line, equilibrium_phases_line, ph_line, time_line, distance_line, simulation_line, high_precision_line, alkalinity_line, solution_line, charge_balance_line, ionic_strength_line, step_line, water_line))
+        
+    def define_paths(self, simulation_name = None, input_path = None, output_path = None):
+        # define the simulation name 
         simulation_number = 1
         if simulation_name is None:
             if self.parameters['simulation_type'] == 'evaporation':
@@ -564,7 +574,8 @@ class ROSSPkg():
             simulation_number += 1
             simulation_name = re.sub('(\-\d+$)', '', simulation_name)
             simulation_name = '-'.join([simulation_name, str(simulation_number)])
-
+            
+        # define the simulation input path 
         if input_path is None:
             self.parameters['input_file_name'] = 'input.pqi'
             working_directory = os.getcwd()
@@ -574,6 +585,20 @@ class ROSSPkg():
         else:
             self.parameters['input_path'] = input_path
             
+        # define the simulation output path 
+        if output_path is None:
+            self.parameters['output_file_name'] = 'selected_output.pqo'
+            self.parameters['output_path'] = os.path.join(self.simulation_path, self.parameters['output_file_name'])
+        else:
+            self.parameters['output_path'] = output_path 
+            
+            
+        return simulation_name, self.parameters['input_path'], self.parameters['output_path']
+
+    def export(self, simulation_name = None, input_path = None, output_path = None, external_file = False):
+        """View and export the PHREEQC input file"""    
+        simulation_name, input_path, output_path = self.define_paths(simulation_name, input_path, output_path)
+            
         # comment the corresponding simulation in the input file
         simulation_line = f'# {self.simulation_path}'
         if not external_file:
@@ -581,6 +606,7 @@ class ROSSPkg():
             self.results['complete_lines'] = chain(self.results['general_conditions'], self.results['solution_block'], self.results['equilibrium_phases_block'], self.results['reaction_block'], self.results['selected_output_block'], self.results['transport_block']) 
             
             # printing and exporting the input file
+            print('\n\n')
             with open(self.parameters['input_path'],'w') as input_file:
                 for line in self.results['complete_lines']:
                     if self.verbose:
@@ -588,14 +614,7 @@ class ROSSPkg():
                     input_file.write(line + '\n')
             
             self.input_file = open(self.parameters['input_path'],'r').read()
-        
-        # define the simulation output path 
-        if output_path is None:
-            self.parameters['output_file_name'] = 'selected_output.pqo'
-            self.parameters['output_path'] = os.path.join(self.simulation_path, self.parameters['output_file_name'])
-        else:
-            self.parameters['output_path'] = output_path    
-        
+
         # define a table of parameters
         parameters = {'parameter':[], 'value':[]}
         parameters['parameter'].append('simulation_path')
@@ -605,7 +624,7 @@ class ROSSPkg():
             parameters['value'].append(self.parameters[parameter])
         parameters_table = pandas.DataFrame(parameters)
         if self.verbose:
-            if jupyter:
+            if self.jupyter:
                 display(parameters_table)
             else:
                 print(parameters_table)
@@ -622,7 +641,7 @@ class ROSSPkg():
             variables['value'].append(self.variables[variable])
         variables_table = pandas.DataFrame(variables)
         if self.verbose:
-            if jupyter:
+            if self.jupyter:
                 display(variables_table)
             else:
                 print(variables_table)
@@ -630,6 +649,13 @@ class ROSSPkg():
         variables_path = os.path.join(self.simulation_path, 'variables.csv')
         variables_table.to_csv(variables_path)
         
+        # export the predicted effluent concentrations
+        effluent_concentrations = pandas.DataFrame(list(self.predicted_effluent.items()), columns = ['elements', 'ppm conc'])
+        effluent_concentrations.index = effluent_concentrations['elements']
+        del effluent_concentrations['elements']
+        effluent_path = os.path.join(self.simulation_path, 'effluent_predictions.csv')
+        effluent_concentrations.to_csv(effluent_path)
+        
     def parse_input(self, input_file_path, simulation, water_selection = None, simulation_name = None, active_feed_area = None):        
         # open the input file
         with open(input_file_path, 'r') as file:
@@ -668,12 +694,12 @@ class ROSSPkg():
                 self.parameters['timestep'] = float(re.sub('-time_step\s+', '', row).split('\t')[0])
                 self.parameters['simulation_time'] = self.parameters['timestep'] * self.simulation_shifts
             elif re.search('-punch_cells', row):
-                row = re.sub('-punch_cells ','',row)
+                row = re.sub('-punch_cells\s+','',row)
                 row = float(re.sub('(\-\d+)', '', row))
                 if row != 1:
                     self.parameters['domain_phase'] = 'Immobile'
             elif re.search('-punch_frequency', row):
-                row = float(re.sub('-punch_frequency ','',row))
+                row = float(re.sub('-punch_frequency\s+','',row))
                 if row > 1:
                     self.parameters['simulation_perspective'] = 'all_distance'               
                 elif row == 1:
@@ -721,7 +747,7 @@ class ROSSPkg():
             headers = conc.keys()
             self.results['csv_data'] = pandas.DataFrame(conc, columns = headers)
             if self.verbose:
-                if jupyter:
+                if self.jupyter:
                     pandas.set_option('display.max_columns', None)
                     display(self.results['csv_data'])
                 else:
@@ -750,7 +776,7 @@ class ROSSPkg():
             
         return self.results['csv_data']
 
-    def process_selected_output(self, selected_output_path = None, plot_title = None, title_font = 'xx-large', label_font = 'x-large', plot_caption = '', legend_title = None, x_label_number = 6, export_name = None, export_format = 'svg', individual_plots = None):
+    def process_selected_output(self, selected_output_path = None, plot_title = None, title_font = 'xx-large', label_font = 'x-large', x_label_number = 6, export_name = None, export_format = 'svg', individual_plots = None):
         """Interpreting the PHREEQC SELECTED_OUTPUT file and conducting the plotting functions"""
         databases = [re.search('(\w+)(?=.json)', database).group() for database in glob('./databases/*.json')]
         if plot_title is not None:
@@ -806,14 +832,14 @@ class ROSSPkg():
                                  
         # conducting the appropriate visualization function
         if self.parameters['simulation'] == 'brine':
-            data = self.brine_plot(pyplot, title_font, label_font, plot_caption, export_format, x_label_number)
+            data = self.brine_plot(pyplot, title_font, label_font, export_format, x_label_number, export_name)
         elif self.parameters['simulation'] == 'scaling':
-            data = self.scaling_plot(title_font, label_font, plot_caption, individual_plots, legend_title, x_label_number, export_name, export_format)
+            data = self.scaling_plot(title_font, label_font, individual_plots, x_label_number, export_name, export_format)
         else:
             print('--> ERROR: The < simulation_perspective > parameter is unpredicted.')
         return data
                                  
-    def brine_plot(self, pyplot, title_font, label_font, plot_caption, export_format, x_label_number = 6):
+    def brine_plot(self, pyplot, title_font, label_font, export_format, x_label_number, export_name):
         """Generate plots of the elemental concentrations from effluent brine in the PHREEQC SELECTED_OUTPUT file"""
         # determine the minerals in the simulation      
         columns = []
@@ -898,7 +924,7 @@ class ROSSPkg():
         legend = non_zero_elements
         legend_title = 'non-zero elements'
         pyplot.yscale('log')
-        mineral = export_name = None
+        mineral = None
         self.illustrate(pyplot, legend, legend_title, mineral, export_name, label_font, title_font, export_format)
 
         # defining the datatable of brine concentrations
@@ -906,13 +932,13 @@ class ROSSPkg():
         concentrations_table.index.name = x_label
         concentrations_table.to_csv(os.path.join(self.simulation_path, 'brine_concentrations.csv'))
         if self.verbose:
-            if jupyter:
+            if self.jupyter:
                 display(concentrations_table)
             else:
                 print(concentrations_table)
         return concentrations_table            
 
-    def scaling_plot(self, title_font, label_font, plot_caption, individual_plots, legend_title = None, x_label_number = 6, export_name = None, export_format = 'svg'):
+    def scaling_plot(self, title_font, label_font, individual_plots, x_label_number, export_name, export_format = 'svg'):
         """Generate plots of scaling along the module distance in the PHREEQC SELECTED_OUTPUT file  """
         # define reused functions
         def legend_determination(time, mineral, mineral_formula):
@@ -1149,19 +1175,22 @@ class ROSSPkg():
         figure = pyplot.gcf()
         pyplot.show()
         self.export_plot(figure, mineral, export_name, export_format)
+        
+    def figure_name(self):
+        # define the output name
+        if self.parameters['simulation'] == 'scaling':
+            if self.parameters['individual_plots']:
+                export_name = mineral
+            else:
+                export_name = 'all_minerals'
+        if self.parameters['simulation'] == 'brine':
+            export_name = 'brine'
+        return export_name
 
     def export_plot(self, figure, mineral = None, export_name = None, export_format = 'svg'):
         """Export the plots to the current working directory  """
-        # define the output name
         if export_name is None:
-            if self.parameters['simulation'] == 'scaling':
-                if self.parameters['individual_plots']:
-                    export_name = mineral
-                else:
-                    export_name = 'all_minerals'
-            if self.parameters['simulation'] == 'brine':
-                export_name = 'brine'
-            
+            export_name = self.figure_name()
         self.results['figures'][export_name] = {'figure':figure, 'title':self.plot_title}
                                 
         # export the plot
